"""A demonstration of a converter written in pure Python"""

from icecube import tableio, dataclasses
from icecube.filterscripts import filter_globals

name = 'OnlineL2'

class BestFitEnum(tableio.enum):
    MPEFit = 0
    SPE2itFit = 1
    PoleMuonLlhFit = 2
    NONE = 3

class BestFitNameConverter(tableio.I3Converter):
    booked = dataclasses.I3String
    def CreateDescription(self, string):
        desc = tableio.I3TableRowDescription()
        desc.add_field('best_fit', tableio.I3Datatype(BestFitEnum), '', 'best fit that was found to succeed')
        return desc
    def FillRows(self, string, rows):
        if string.value == name+'_MPEFit':
            rows['best_fit'] = BestFitEnum.MPEFit
        elif string.value == name+'_SPE2itFit':
            rows['best_fit'] = BestFitEnum.SPE2itFit
        elif string.value == filter_globals.muon_llhfit:
            rows['best_fit'] = BestFitEnum.PoleMuonLlhFit
        else:
            rows['best_fit'] = BestFitEnum.NONE
        return 1 # return number of rows added (only one in this case)

tableio.I3ConverterRegistry.register(BestFitNameConverter)
