from icecube import icetray, dataio, dataclasses
from icecube import phys_services, filter_tools
import filter_globals
#import all the actual worker segments....
from OnlineL2Filter import OnlineL2Filter
from OpticalFollowUp import OpticalFollowUp

@icetray.traysegment
def OnlineFilter(tray, name,
        PhotonicsTabledirMu = None, PhotonicsDriverfileMu_Spice1 = None, PhotonicsDriverdirMu_Spice1 = None,
        SplineRecoPhotosplineTableDir = None, bdtfile = None):

    pulses = "OnlineL2_"+filter_globals.CleanedMuonPulses # those are the
                                # pulses that were used originally online

# Filter on OnlineL2 events:
    def eventHasPassedOnlineL2(frame):
        return frame.Has(pulses)

    # list of things that will be re-created by OnlineL2, so rename them to avoid conflicts
    # (they will be kept for later comparison)
    to_rename_ol2_with_prefix = [  # Reco results from online L2
            "OnlineL2_BayesianFit",
            "OnlineL2_BayesianFitFitParams",
            "OnlineL2_BestFit",
            "OnlineL2_BestFitCharacteristics",
            "OnlineL2_BestFitCharacteristicsWithSAll",
            "OnlineL2_BestFitDirectHitsA",
            "OnlineL2_BestFitDirectHitsB",
            "OnlineL2_BestFitDirectHitsC",
            "OnlineL2_BestFitDirectHitsD",
            "OnlineL2_BestFitFitParams",
            "OnlineL2_BestFitName",
            "OnlineL2_BestFit_MuE",
            "OnlineL2_BestFit_MuEx",
            "OnlineL2_BestFit_ParaboloidFitParams",
            "OnlineL2_BestFit_TruncatedEnergy_AllBINS_MuEres",
            "OnlineL2_BestFit_TruncatedEnergy_AllBINS_Muon",
            "OnlineL2_BestFit_TruncatedEnergy_AllBINS_Neutrino",
            "OnlineL2_BestFit_TruncatedEnergy_AllDOMS_MuEres",
            "OnlineL2_BestFit_TruncatedEnergy_AllDOMS_Muon",
            "OnlineL2_BestFit_TruncatedEnergy_AllDOMS_Neutrino",
            "OnlineL2_BestFit_TruncatedEnergy_BINS_MuEres",
            "OnlineL2_BestFit_TruncatedEnergy_BINS_Muon",
            "OnlineL2_BestFit_TruncatedEnergy_BINS_Neutrino",
            "OnlineL2_BestFit_TruncatedEnergy_DOMS_MuEres",
            "OnlineL2_BestFit_TruncatedEnergy_DOMS_Muon",
            "OnlineL2_BestFit_TruncatedEnergy_DOMS_Neutrino",
            "OnlineL2_BestFit_TruncatedEnergy_ORIG_Muon",
            "OnlineL2_BestFit_TruncatedEnergy_ORIG_Neutrino",
            "OnlineL2_BestFit_TruncatedEnergy_ORIG_dEdX",
            "OnlineL2_CramerRao_BestFitParams",
            "OnlineL2_CramerRao_BestFit_cr_azimuth",
            "OnlineL2_CramerRao_BestFit_cr_zenith",
            "OnlineL2_CramerRao_MPEFitParams",
            "OnlineL2_CramerRao_MPEFit_cr_azimuth",
            "OnlineL2_CramerRao_MPEFit_cr_zenith",
            "OnlineL2_CramerRao_SPE2itFitParams",
            "OnlineL2_CramerRao_SPE2itFit_cr_azimuth",
            "OnlineL2_CramerRao_SPE2itFit_cr_zenith",
            "OnlineL2_CramerRao_SplineMPEParams",
            "OnlineL2_CramerRao_SplineMPE_cr_azimuth",
            "OnlineL2_CramerRao_SplineMPE_cr_zenith",
            "OnlineL2_HitMultiplicityValues",
            "OnlineL2_HitStatisticsValues",
            "OnlineL2_MPEFit",
            "OnlineL2_MPEFitDirectHitsA",
            "OnlineL2_MPEFitDirectHitsB",
            "OnlineL2_MPEFitDirectHitsC",
            "OnlineL2_MPEFitDirectHitsD",
            "OnlineL2_MPEFitFitParams",
            "OnlineL2_SPE2itFit",
            "OnlineL2_SPE2itFitDirectHitsA",
            "OnlineL2_SPE2itFitDirectHitsB",
            "OnlineL2_SPE2itFitDirectHitsC",
            "OnlineL2_SPE2itFitDirectHitsD",
            "OnlineL2_SPE2itFitFitParams",
            "OnlineL2_SplineMPE",
            "OnlineL2_SplineMPEFitParams",
            "OnlineL2_SplitGeo1_BayesianFit",
            "OnlineL2_SplitGeo1_BayesianFitFitParams",
            "OnlineL2_SplitGeo1_Linefit",
            "OnlineL2_SplitGeo1_SPE2itFit",
            "OnlineL2_SplitGeo1_SPE2itFitFitParams",
            "OnlineL2_SplitGeo2_BayesianFit",
            "OnlineL2_SplitGeo2_BayesianFitFitParams",
            "OnlineL2_SplitGeo2_Linefit",
            "OnlineL2_SplitGeo2_SPE2itFit",
            "OnlineL2_SplitGeo2_SPE2itFitFitParams",
            "OnlineL2_SplitTime1_BayesianFit",
            "OnlineL2_SplitTime1_BayesianFitFitParams",
            "OnlineL2_SplitTime1_Linefit",
            "OnlineL2_SplitTime1_SPE2itFit",
            "OnlineL2_SplitTime1_SPE2itFitFitParams",
            "OnlineL2_SplitTime2_BayesianFit",
            "OnlineL2_SplitTime2_BayesianFitFitParams",
            "OnlineL2_SplitTime2_Linefit",
            "OnlineL2_SplitTime2_SPE2itFit",
            "OnlineL2_SplitTime2_SPE2itFitFitParams",
            ]
    renamed_ol2 = [ "OnlineL2_BadGCD_"+key[9:] for key in to_rename_ol2_with_prefix ]
    to_rename_ol2_no_prefix = [
            "PoleMuonLlhFitDirectHitsA",
            "PoleMuonLlhFitDirectHitsB",
            "PoleMuonLlhFitDirectHitsC",
            "PoleMuonLlhFitDirectHitsD",
            ]
    renamed_ol2 += [ "OnlineL2_BadGCD_"+key for key in to_rename_ol2_no_prefix ]
    to_rename_ol2 = to_rename_ol2_with_prefix + to_rename_ol2_no_prefix
    rename_pairs_ol2 = [ key for keys in zip(to_rename_ol2, renamed_ol2) for key in keys ]
    tray.AddModule("Rename", name+"_RenameOnlineL2", Keys=rename_pairs_ol2,
            If = eventHasPassedOnlineL2
            )

    # list of things that will be re-created by OpticalFollowUp, so rename them to avoid conflicts
    # (they will be kept for later comparison)
    to_rename_ofu = [
            "OpticalFollowUp_valuesdict",
            "OpticalFollowUp_BDT_Score",
            ]
    renamed_ofu = [ "OpticalFollowUp_BadGCD_"+key[16:] for key in to_rename_ofu ]
    rename_pairs_ofu = [ key for keys in zip(to_rename_ofu, renamed_ofu) for key in keys ]
    tray.AddModule("Rename", name+"_RenameOFU", Keys=rename_pairs_ofu,
            If = eventHasPassedOnlineL2
            )

# Put bool in frame, only if it's true, event will be re-processed
# by OnlineL2Filter (originally all events passing MuonFilter):
# Now only events having the required online pulses (events that passed OnlineL2 at pole)
    def PutProcessOnlineL2BoolInFrame(frame):
        if eventHasPassedOnlineL2(frame):
            frame.Delete(filter_globals.MuonFilter) # just in case, bool shouldn't be there
            frame[filter_globals.MuonFilter] = icetray.I3Bool( eventHasPassedOnlineL2(frame) )
    tray.AddModule(PutProcessOnlineL2BoolInFrame, name+"_PutProcessOnlineL2BoolInFrame")

## OnlineL2
    tray.AddSegment(OnlineL2Filter, "OnlineL2",
                    pulses = pulses,
                    llhfit_name = filter_globals.muon_llhfit,
                    angular_error=True, spline=True,
                    PhotonicsTabledirMu = PhotonicsTabledirMu,
                    PhotonicsDriverdirMu_Spice1 = PhotonicsDriverdirMu_Spice1,
                    PhotonicsDriverfileMu_Spice1 = PhotonicsDriverfileMu_Spice1,
                    SplineDir = SplineRecoPhotosplineTableDir,
                    If = eventHasPassedOnlineL2
                    )
## OpticalFollowUp
    tray.AddSegment(OpticalFollowUp, "OpticalFollowUp",
                    OnlineL2SegmentName = "OnlineL2",
                    bdtfile = bdtfile,
                    testing = False,
                    If = eventHasPassedOnlineL2
                    )

## FilterMask needs to be updated
    def DecodeFilterMask(frame, filterMaskName="FilterMask"):
        '''
        Put each FilterMask decision as Bool into frame.
        '''
        if frame.Has(filterMaskName):
            frame.Delete(filter_globals.MuonFilter) # clean this (filled before) so that there is room for actual MuonFilter bool
            fmask = frame[filterMaskName]
            for filter in fmask.keys():
# old versions of OnlineL2Filter and OFUFilter are recovered under different
# name (so that new version is not overwritten)
                if filter == filter_globals.OnlineL2Filter or filter == filter_globals.OFUFilter:
                    frame[filter+"_BadGCD"] = icetray.I3Bool( fmask[filter].condition_passed )
                else:
                    frame[filter] = icetray.I3Bool( fmask[filter].condition_passed )
        return True
    tray.AddModule(DecodeFilterMask, name+"_FilterMaskDecode", filterMaskName=filter_globals.filter_mask)

# Store precious old QFilterMask for recovery in case of new empty QFilterMask
# and store old FilterMask in order to take over the old prescale_passed values
# that are random generated and would otherwise be changed by reprocessing
    tray.AddModule("Rename", name+"_RenameOldQFilterMask",
            Keys=[filter_globals.qfilter_mask, "Original_QFilterMask",
                  filter_globals.filter_mask, "Original_FilterMask"],
            )

# add the old versions of OnlineL2Filter and OFUFilter to the filter mask as well:
    filter_globals.filter_pairs += [
            (filter_globals.OnlineL2Filter+"_BadGCD",1),
            (filter_globals.OFUFilter+"_BadGCD",1),
            ("I3DAQDecodeException",1),
            ("LID",1),
            ]

# Generate new FilterMask for all P frames
# Do this *AFTER* the filters run, in order to update the FilterMask
# with the results
    filter_mask_randoms = phys_services.I3GSLRandomService(9999)
    print filter_globals.filter_pairs + filter_globals.sdst_pairs
    tray.AddModule(filter_tools.FilterMaskMaker, name+"_MakeFilterMasks",
            OutputMaskName = filter_globals.filter_mask,
            FilterConfigs = filter_globals.filter_pairs + filter_globals.sdst_pairs,
            RandomService = filter_mask_randoms,
            If = lambda frame: frame.Has(filter_globals.OnlineL2Filter+"_BadGCD") # only if DecodeFilterMask has run
            )

# Take the original prescale_passed vaues from the original FilterMask and put
# them into the new FilterMask
    def adoptFilterMaskPrescaleValues(frame, OriginalFilterMask="", NewFilterMask=""):
        if frame.Has(OriginalFilterMask) and frame.Has(NewFilterMask):
            ofm = frame[OriginalFilterMask]
            nfm = frame[NewFilterMask]
            for filter in ofm.keys():
                nfm[filter].prescale_passed = ofm[filter].prescale_passed
            frame.Delete(NewFilterMask)
            frame[NewFilterMask] = nfm
            frame.Delete(OriginalFilterMask)
    tray.AddModule(adoptFilterMaskPrescaleValues, name+"_adopt_prescale_values",
            OriginalFilterMask = "Original_FilterMask",
            NewFilterMask = filter_globals.filter_mask,
            )

# Merge the FilterMasks into Q frame:
    tray.AddModule("OrPframeFilterMasks", name+"_make_q_filtermask",
            InputName = filter_globals.filter_mask,
            OutputName = filter_globals.qfilter_mask,
            )

    class OrOldAndNewQFilterMask(icetray.I3PacketModule):
        def __init__(self, context):
            icetray.I3PacketModule.__init__(self, context, icetray.I3Frame.DAQ)
            #self.AddParameter("offset", "Offset at start of i3file", 1)
            self.AddOutBox("OutBox")
        def Configure(self):
            #self.offset = self.GetParameter("offset")
            pass
        def FramePacket(self, frames):
            qframe = frames[0] # first frame should always be Q frame
            old_qfmask = qframe["Original_QFilterMask"]
            new_qfmask = qframe[filter_globals.qfilter_mask]
            for filter in set( old_qfmask.keys() + new_qfmask.keys() ):
                fr = dataclasses.I3FilterResult()
                if filter in old_qfmask:
                    old_c_passed = old_qfmask[filter].condition_passed
                    old_p_passed = old_qfmask[filter].prescale_passed
                else:
                    old_c_passed = False
                    old_p_passed = False
                if filter in new_qfmask:
                    new_c_passed = new_qfmask[filter].condition_passed
                    new_p_passed = new_qfmask[filter].prescale_passed
                else:
                    new_c_passed = False
                    new_p_passed = False
                # OR the original and new QFilterMask
                fr.condition_passed = old_c_passed or new_c_passed
                fr.prescale_passed = old_p_passed or new_p_passed
                new_qfmask[filter] = fr
            # Write the OR'd QFilterMask to the frame
            qframe.Delete(filter_globals.qfilter_mask)
            qframe[filter_globals.qfilter_mask] = new_qfmask
            qframe.Delete("Original_QFilterMask") # delete the original mask
            # push all the frames:
            for frame in frames:
                self.PushFrame(frame)
    tray.AddModule(OrOldAndNewQFilterMask, name+"_recover_q_filtermask")


# Clean up temporary bools
    bools_list = [ p[0] for p in filter_globals.filter_pairs+filter_globals.sdst_pairs ]
    tray.AddModule("Delete", name+"_DeleteTempBools", Keys=bools_list)

# Clean up other leftovers
    clean_keys = [
            "OnlineL2_BestFit_TruncatedEnergy_AllBINS_dEdX",
            "OnlineL2_BestFit_TruncatedEnergy_AllDOMS_dEdX",
            "OnlineL2_BestFit_TruncatedEnergy_BINS_dEdX",
            "OnlineL2_BestFit_TruncatedEnergy_DOMS_dEdX",
            "OnlineL2_SplitGeo1_LinefitParams",
            "OnlineL2_SplitGeo2_LinefitParams",
            "OnlineL2_SplitTime1_LinefitParams",
            "OnlineL2_SplitTime2_LinefitParams",
            "OnlineL2_BestFit_Paraboloid",
            ]
    tray.AddModule("Delete", name+"_DeleteOtherLeftovers", Keys=clean_keys,
            If = eventHasPassedOnlineL2
            )

