#!/usr/bin/env python


from I3Tray import *
import sys, os, glob
from icecube import icetray, dataclasses, dataio, filter_tools, phys_services
import filter_globals
from filter_globals import which_split
from OnlineFilter import OnlineFilter


sys.path.append("/data/user/i3filter/IC86_OfflineProcessing/OfflineProductionTools")
sys.path.append("/data/user/i3filter/SQLServers_n_Clients")

from RunTools import *
from DbTools import *
from FileTools import *


try:
    import SQLClient_dbs4 as dbs4
    dbs4_ = dbs4.MySQL()
    
except Exception, err:
    raise Exception("Error: %s "%str(err))


def make_parser():
    """Make the argument parser"""
    from optparse import OptionParser
    parser = OptionParser()
    
    parser.add_option("-i", "--input", action="store",
        type="string", default="", dest="infile",
        help="Input i3 file(s)  (use comma separated list for multiple files)")
    
    parser.add_option("-g", "--gcd", action="store",
        type="string", default="", dest="gcdfile",
        help="GCD file for input i3 file")
    
    parser.add_option("-o", "--output", action="store",
        type="string", default="", dest="outfile",
        help="Main i3 output file")
    
    parser.add_option("--onlinel2", action="store_true", default=False,
                  dest="ONLINEL2", help="Filter to OnlineL2 level.")

    parser.add_option("--photonicsdir", action="store",
        type="string", default="/cvmfs/icecube.wisc.edu/data/photon-tables/SPICE1/",
        dest="photonicsdir", help="Directory with photonics tables")
    
    parser.add_option("--photonicsdriverdir", action="store",
        type="string", default="/cvmfs/icecube.wisc.edu/data/photon-tables/SPICE1/driverfiles",
        dest="photonicsdriverdir", help="Directory with photonics tables driver files")
    
    parser.add_option("--photonicsdriverfile", action="store",
        type="string", default="mu_photorec.list",
        dest="photonicsdriverfile", help="photonics driver file")
    
    #parser.add_option("--splinedir", action="store",
    #    type="string", default="/cvmfs/icecube.wisc.edu/data/photon-tables/splines/",
    #    dest="splinedir", help="Spline Dir")
    parser.add_option("--splinedir", action="store",
        type="string", default="/data/sim/sim-new/downloads/IC86_2014_OnlineReprocessingData/InfBareMu_mie_spline/",
        dest="splinedir", help="Spline Dir")
    
    parser.add_option("--bdtfile", action="store",
        type="string", default= "/data/exp/IceCube/2014/filtered/level2/optical-follow-up-filter_V14-04-01/resources/bdts/bdt_ofu_IC86_2.bdt",
        dest="bdtfile", help="location of bdt file")
    
    parser.add_option("-n", "--num", action="store",
        type="int", default=-1, dest="num",
        help="Number of frames to process")
    
    
    return parser 


#def main(params):
def main(options, stats={}):
    icetray.logging.set_level("WARN")

    tray = I3Tray()
    
    files = [options['gcdfile']]

    if isinstance(options['infile'],str):
        files.append(options['infile'])
    else:
        for f in options['infile']:files.append(f)

    
    tray.AddModule("I3Reader", "reader", filenamelist=files)

    tray.AddSegment(OnlineFilter, "OnlineReprocess",
                   PhotonicsTabledirMu=options['photonicsdir'],
                   PhotonicsDriverdirMu_Spice1 = options['photonicsdriverdir'],
                   PhotonicsDriverfileMu_Spice1 = options['photonicsdriverfile'],
                   SplineRecoPhotosplineTableDir = options['splinedir'],
                   bdtfile = options['bdtfile']
                   )

    def filterMaskPick(frame, filterName, filterMaskName="FilterMask"):
        if frame.Has(filterMaskName):
            fmask = frame[filterMaskName]
            if filterName in fmask.keys():
                return fmask[filterName].condition_passed
        return False

    if options['ONLINEL2']:
        print 'Will filter down to OnlineL2 level.'
        
        tray.AddModule(filterMaskPick, "OnlineL2FilterPick",
                filterName = filter_globals.OnlineL2Filter,
                filterMaskName = filter_globals.filter_mask
                )
    else:
        print 'Will keep all events.'

    ## Write the physics and DAQ frames
    tray.AddModule( "I3Writer", "EventWriter", filename=options['outfile'],
                        Streams=[icetray.I3Frame.Physics,icetray.I3Frame.DAQ],
                        DropOrphanStreams = [icetray.I3Frame.DAQ],
                        )

    tray.AddModule("TrashCan","bye")
    
    # make it go
    if options['num'] >= 0:
        tray.Execute(options['num'])
    else:
        tray.Execute()
    
    tray.Finish()
    



### iceprod stuff ###
try:
    from iceprod.modules import ipmodule
except ImportError, e:
    print 'Module iceprod.modules not found. Will not define IceProd Class'
else:
    Level2Filter = ipmodule.FromOptionParser(make_parser(),main)
### end iceprod stuff ###
    


# the business!
if (__name__=="__main__"):

    from optparse import OptionParser

    parser = make_parser()
    
    (options,args) = parser.parse_args()
    opts = {}
    # convert to dictionary
    for name in parser.defaults:
        value = getattr(options,name)
        if name == 'infile' and ',' in value:
            value = value.split(',') # split into multiple inputs
        opts[name] = value

    print "\nArgument : Value"
    for k in opts:
        print k," : ",opts[k]
    
    if opts['infile']=='' or opts['gcdfile']=='' or opts['outfile']=='':
        print "you must specify the input, gcd and output file arguments"
        exit(1)

    main(opts)

