#
# Set of common definitions used by all filters.  Name definitions mostly
#
from icecube import icetray, dataclasses

#
# functions to conditionally execute based on split
#
InIceSplitter = 'InIceSplit'
IceTopSplitter = 'IceTopSplit'
NullSplitter = 'NullSplit'
SLOPSplitter = 'SLOPSplit'

def print_split(frame):
    if frame.Stop == icetray.I3Frame.Physics:
        print frame['I3EventHeader'].sub_event_stream


def which_split(split_name = None):
    def which_split(frame):
        if split_name == None:
            print "Select a split name in you If...Running module anyway"
            return 1
        if frame.Stop == icetray.I3Frame.Physics:
            return frame['I3EventHeader'].sub_event_stream == split_name
        else:
            return 0
    return which_split


#
#DOMLaunches
#

#InIce Bad dom cleaned list:
CleanInIceRawData = 'CleanInIceRawData'

#IceTop Bad dom cleaned list:
CleanIceTopRawData = 'CleanIceTopRawData'

#
# RecoPulseSeries/Maps
#
UncleanedInIcePulses       = 'UncleanedInIcePulses'
SplitUncleanedInIcePulses  = 'SplitUncleanedInIcePulses'
SplitRTCleanedInIcePulses  = 'SplitRTCleanedInIcePulses'
SplitUncleanedITPulses  = 'SplitUncleanedITPulses'
SLOPPulses = 'SLOPPulseMask'

IceTopPulses_HLC = 'IceTopPulses_HLC'
IceTopPulses_SLC = 'IceTopPulses_SLC'
HLCTankPulses = 'HLCTankPulses'
CleanedHLCTankPulses = 'CleanedHLCTankPulses'
IceTopBadDoms = 'IceTopBadDoms'
IcetopBadTanks = 'IceTopBadTanks'
TankPulseMergerExcludedTanks = 'TankPulseMergerExcludedTanks'
ClusterCleaningExcludedTanks = 'ClusterCleaningExcludedTanks'
IceTopRecoStandard = 'LaputopStandard'
IceTopRecoSmallShowers = 'LaputopSmallShower'
IsSmallShower = 'IsSmallShower'

DSTPulses = 'DSTPulses'
InIceDSTPulses = 'InIce' + DSTPulses
IceTopDSTPulses = 'IceTop' + DSTPulses

CleanedMuonPulses = 'CleanedMuonPulses'


# trigger I3Bools from TriggerChecker
inicesmttriggered = 'InIceSMTTriggered'
icetopsmttriggered = 'IceTopSMTTriggered'
inicestringtriggered = 'InIceStringTriggered'
deepcoresmttriggered = 'DeepCoreSMTTriggered'
volumetrigtriggered = 'VolumeTriggerFlag'
slowparticletriggered = 'SlowParticleTriggered'
fixedratetriggered = 'FixedRateTriggered'

#TODO: make sure configIDs have not changed in 2013,
# commit any changes at last minute for test run
# The triggerConfigIDs that are specified.
deepcoreconfigid = 1011
inicesmtconfigid = 1006
inicestringconfigid = 1007
slowparticleconfigid = 24002
volumetriggerconfigid = 21001
icetopsmtconfigid = 102
fixedrateconfigid = 23051

#
# Shared reconstructions:
#
muon_linefit = 'PoleMuonLinefit'
muon_llhfit = 'PoleMuonLlhFit'

## I3Doubles for the cascaders -- does this belong here?
cascade_toi = 'PoleCascadeFilter_ToiVal'
cascade_lfvel = 'PoleCascadeFilter_LFVel'
cascade_cscdllh = 'PoleCascadeFilter_CscdLlh'


## standard items
rawdaqdata = 'I3DAQData'
seatbeltdata = 'I3DAQDataTrimmed'
icetoprawdata = 'I3DAQDataIceTop'
superdst = 'I3SuperDST'
eventheader = 'I3EventHeader'
triggerhierarchy = 'I3TriggerHierarchy'
qtriggerhierarchy = 'QTriggerHierarchy'
smalltriggerhierarchy = 'DSTTriggers'
dst  = 'I3DST13'
dstreco  = 'I3DST13Reco'
dstheader  = 'I3DST13Header'

filter_mask = 'FilterMask'
qfilter_mask = 'QFilterMask'
icetop_vemcal = 'I3VEMCalData'
#
# FilterNames
#
FilterMinBias = 'FilterMinBias_13'
SDST_FilterMinBias = 'SDST_FilterMinBias_13'
MuonFilter = 'MuonFilter_13'
CosmicRayFilter = 'CosmicRayFilter_13'
SunFilter = 'SunFilter_13'
MoonFilter = 'MoonFilter_13'
CascadeFilter = 'CascadeFilter_13'
EHEFilter = 'EHEFilter_13'
OnlineL2Filter = 'OnlineL2Filter_14'
OFUFilter = 'OFUFilter_14'
DeepCoreFilter = 'DeepCoreFilter_13'
DeepCoreFilter_TwoLayerBranch = 'DeepCoreFilter_TwoLayerExp_13'
FSSFilter = 'FSSFilter_13'
FSSCandidate = 'FSSCandidate_13'
GCFilter = 'GCFilter_13'
VEFFilter = 'VEF_13'
LowUpFilter = 'LowUp_13'
SlopFilter = 'SlopFilter_13'
FixedRateFilterName = 'FixedRateFilter_13'

filter_pairs = [(FilterMinBias,1000),
                (MuonFilter,1),
                (MoonFilter,1),
                (SunFilter,1),
                (OnlineL2Filter,1),
                (OFUFilter,1),
                (CascadeFilter,1),
                (EHEFilter,1),
                ('IceTopSTA3_13',10),
                ('IceTopSTA5_13',1),
                ('IceTop_InFill_STA3_13',10),
                ('InIceSMT_IceTopCoincidence_13',100),
                (DeepCoreFilter,1),
                (DeepCoreFilter_TwoLayerBranch,1),
                (FSSFilter,1),
                (FSSCandidate,0),
                (GCFilter,1),
                (VEFFilter,1),
                (LowUpFilter,1),
                (SlopFilter, 1),
                (FixedRateFilterName,1)
                ]

filter_streams,filter_prescales = zip(*filter_pairs)

## Filters that select events for "SuperDST only" transmission
sdst_pairs = [(SDST_FilterMinBias,5),
              ('SDST_InIceSMT_IceTopCoincidence_13',1),
              ('SDST_IceTopSTA3_13',1),
              ('SDST_IceTop_InFill_STA3_13',1)]

sdst_streams,sdst_prescales = zip(*sdst_pairs)

## List of filters that want *ALL* waveforms as well as SuperDST
filters_keeping_allraw = [FilterMinBias,
                          'IceTopSTA3_13',
                          'IceTopSTA5_13',
                          'IceTop_InFill_STA3_13',
                          'InIceSMT_IceTopCoincidence_13',
                          EHEFilter,
                          FixedRateFilterName]

q_frame_keeps = [smalltriggerhierarchy,
                  seatbeltdata,
                  icetoprawdata,
                  superdst,
                  eventheader,
                  'JEBClientInfo',
                  'PFClientInfo',
                  qfilter_mask,
                  dst,
                  dstheader,
                  icetop_vemcal]

sdstall_keeps = [superdst,
                 seatbeltdata,
                 smalltriggerhierarchy,
                 icetoprawdata]

null_split_keeps = [filter_mask]

inice_split_keeps = [filter_mask,
                      muon_linefit,
                      muon_llhfit,
                      muon_llhfit+'FitParams',
                      cascade_toi,
                      cascade_lfvel,
                      cascade_cscdllh,
                      'CorsikaMoonMJD', # only affects simulation: saves an I3Double with the generated fake MJD
                      'CorsikaSunMJD', # only affects simulation: saves an I3Double with the generated fake MJD
                      dstreco,
                      'PoleEHESummaryPulseInfo', # EHE info
                     ]

icetop_split_keeps = [filter_mask,
                      IceTopRecoStandard,
                      IceTopRecoSmallShowers,
                      IceTopRecoStandard + 'Params',
                      IceTopRecoSmallShowers + 'Params',
                      IsSmallShower,]

slop_split_keeps = [filter_mask,
                    'SLOPTuples_CosAlpha_Launches',
                    'SLOPTuples_RelV_Launches',
                    'SLOPLaunchMapTuples']

keep_nofilterpass = [smalltriggerhierarchy,
                     superdst,
                     eventheader,
                     'JEBClientInfo',
                     'PFClientInfo',
                     qfilter_mask,
                     dst,
                     dstheader,
                     icetop_vemcal,
                     dstreco]

keep_dst_only = ['JEBClientInfo',
                 'PFClientInfo',
                 dst,
                 dstheader,
                 icetop_vemcal,
                 dstreco]

onlinel2filter_keeps = []  ## added to in l2
ofufilter_keeps = []  ## added to in ofu

